@@ -280,6 +280,13 @@ mergeInto(LibraryManager.library, {
 
         function handleMessage(data) {
           assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
+
+          // An empty ArrayBuffer will emit a pseudo disconnect event
+          // as recv/recvmsg will return zero which indicates that a socket
+          // has performed a shutdown although the connection has not been disconnected yet.
+          if (data.byteLength == 0) {
+            return;
+          }
           data = new Uint8Array(data);  // make a typed array view on the array buffer
 
 #if SOCKET_DEBUG
@@ -692,10 +699,10 @@ mergeInto(LibraryManager.library, {
         if (event === 'error') {
           var sp = Runtime.stackSave();
           var msg = allocate(intArrayFromString(data[2]), 'i8', ALLOC_STACK);
-          Runtime.dynCall('viiii', callback, [data[0], data[1], msg, userData]);
+          Module['dynCall_viiii'](callback, data[0], data[1], msg, userData);
           Runtime.stackRestore(sp);
         } else {
-          Runtime.dynCall('vii', callback, [data, userData]);
+          Module['dynCall_vii'](callback, data, userData);
         }
       } catch (e) {
         if (e instanceof ExitStatus) {
