@@ -438,6 +438,9 @@ extern EM_BOOL emscripten_is_webgl_context_lost(const char *target);
 
 extern EMSCRIPTEN_RESULT emscripten_webgl_commit_frame();
 
+extern EMSCRIPTEN_RESULT emscripten_set_canvas_element_size(const char *target, int width, int height);
+extern EMSCRIPTEN_RESULT emscripten_get_canvas_element_size(const char *target, int *width, int *height);
+
 extern EMSCRIPTEN_RESULT emscripten_set_element_css_size(const char *target, double width, double height);
 extern EMSCRIPTEN_RESULT emscripten_get_element_css_size(const char *target, double *width, double *height);
 
